import { getDevOpsCore } from '@thatdevops/core/devopsCore'
import Pkg from '@thatdevops/core/pkgman/Pkg'
import spawnSilently from '@thatdevops/core/utils/spawnSilently'
import fs from 'fs'
import { DateTime } from 'luxon'
import path from 'path'

export async function doPublishPackage(pkg: Pkg, newVersion: string): Promise<void> {
	const { rootPath } = getDevOpsCore()
	console.log(`- ${pkg.json.name} to ${newVersion}`)

	await spawnSilently('yarn', ['publish', '--new-version', newVersion], pkg.location)

	const now = DateTime.now().toISO()

	const rootJsonPath = path.join(rootPath, 'package.json')
	const rootJson = JSON.parse(fs.readFileSync(rootJsonPath).toString())
	rootJson.devops = rootJson.devops || {}
	rootJson.devops.publishdates = rootJson.devops.publishdates || {}
	rootJson.devops.publishdates[pkg.json.name] = now
	fs.writeFileSync(rootJsonPath, JSON.stringify(rootJson, null, 4))
}
