"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.gitCommitBumpPublish = void 0;
const devopsCore_1 = require("@thatdevops/core/devopsCore");
const getProjectPkgs_1 = __importDefault(require("@thatdevops/core/pkgman/getProjectPkgs"));
const echo_1 = __importDefault(require("@thatdevops/core/utils/echo"));
const makeRunable_1 = require("@thatdevops/core/utils/makeRunable");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const doPublishPackage_1 = require("./_/doPublishPackage");
const versionBump_1 = require("./_/versionBump");
exports.gitCommitBumpPublish = (0, makeRunable_1.makeRunnable)((o) => __awaiter(void 0, void 0, void 0, function* () {
    const { publishable } = (0, devopsCore_1.getDevOpsCore)();
    if (!publishable) {
        (0, echo_1.default)('Publishing not allowed (set the dev ops "publishable" flag');
        return;
    }
    (0, echo_1.default)('Finding changed packages');
    const changedPkgs = (yield (0, getProjectPkgs_1.default)()).filter(p => p.relativePath === 'packages/thatweb/actionkeys');
    const effectedPkgs = yield doFindEffectedPackages(changedPkgs);
    const newVersions = doCalcNewVersions(effectedPkgs);
    (0, echo_1.default)('Updating related dependencies');
    yield doUpdateDependencies(newVersions);
    (0, echo_1.default)('Publishing changed packages');
    for (const pkg of changedPkgs) {
        yield (0, doPublishPackage_1.doPublishPackage)(pkg, newVersions[pkg.name]);
    }
}));
exports.default = exports.gitCommitBumpPublish;
function doCalcNewVersions(pkgs, bumpType = versionBump_1.BumpType.patch) {
    const newVersions = {};
    for (const pkg of pkgs) {
        newVersions[pkg.name] = (0, versionBump_1.versionBump)(pkg.version, bumpType);
    }
    return newVersions;
}
function doUpdateDependencies(newVersions) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkgs = yield (0, getProjectPkgs_1.default)();
        const changed = [];
        for (const pkg of pkgs) {
            for (const depList of ['dependencies', 'devDependencies']) {
                const deps = pkg.json[depList];
                for (const depName in deps) {
                    if (newVersions[depName]) {
                        const oldVersion = deps[depName];
                        const newVersion = newVersions[depName];
                        if (oldVersion !== newVersion) {
                            deps[depName] = newVersion;
                            changed.push(pkg);
                        }
                    }
                }
            }
        }
        for (const pkg of changed) {
            fs_1.default.writeFileSync(path_1.default.join(pkg.location, 'package.json'), JSON.stringify(pkg.json, null, 4));
        }
    });
}
function doFindEffectedPackages(changedPkgs) {
    return __awaiter(this, void 0, void 0, function* () {
        const allEffected = [];
        let newEffected = yield doFindNewEffectedPackages(changedPkgs);
        allEffected.push(...newEffected);
        while (newEffected.length) {
            newEffected = yield doFindNewEffectedPackages(newEffected);
            allEffected.push(...newEffected);
        }
        return allEffected;
    });
}
function doFindNewEffectedPackages(changedPkgs) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkgs = yield (0, getProjectPkgs_1.default)();
        const effected = new Set();
        for (const changedPkg of changedPkgs) {
            effected.add(changedPkg);
            for (const pkg of pkgs) {
                for (const depList of ['dependencies', 'devDependencies']) {
                    const deps = pkg.json[depList];
                    for (const depName in deps) {
                        if (depName === changedPkg.name) {
                            effected.add(pkg);
                        }
                    }
                }
            }
        }
        return Array.from(effected);
    });
}
//# sourceMappingURL=commitPushPublish.js.map